import sys
import argparse
import ast
import subprocess
import os
import astunparse
from typing import List

__all__ = ('main',)


def parseargs(args=None):
    parser = argparse.ArgumentParser(description='Evaluate a Python expression')
    parser.add_argument('imports', nargs='*',
                        help='Modules to import')
    parser.add_argument('expr',
                        help='Expression to evaluate')
    parser.add_argument('-p', '--python', default=sys.executable,
                        help='Python to use')
    parser.add_argument('-e', '--venv', action='store_true',
                        help='Use the active virtual environment')

    return parser.parse_args(args)


def find_binary(bin, use_venv):
    if use_venv and 'VIRTUAL_ENV' in os.environ:
        # FIXME: Do the Windows version of this
        return os.path.join(os.environ['VIRTUAL_ENV'], 'bin', 'python')
    else:
        return bin


def build_imports(modules):
    return [
        ast.ImportFrom(module=mod, names=[ast.alias(name='*', asname=None)], level=0)
        for mod in modules
    ]


def build_print_repr(expr):
    return [
        ast.Expr(
            value=ast.Call(
                func=ast.Name(id='print', ctx=ast.Load()), 
                args=[ast.Call(
                    func=ast.Name(id='repr', ctx=ast.Load()),
                    args=[expr],
                    keywords=[],
                )],
                keywords=[],
            )
        )
    ]


def recompile(code, imports):
    expr = ast.parse(code, filename='<args>', mode='eval')
    return ast.Module(build_imports(imports) + build_print_repr(expr.body))


def main(args: List[str]=None) -> int:
    opts = parseargs(args)
    bin = find_binary(opts.python, opts.venv)
    tree = recompile(opts.expr, opts.imports)
    code = astunparse.unparse(tree)

    proc = subprocess.run([bin, '-c', code])
    return proc.returncode


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3

import inspect
import sys
import urllib.request

def fetch(method, url):
    method = method.upper()
    request  = urllib.request.Request(url, method=method)
    response = urllib.request.urlopen(request)
    return response

def _main(argv=None):
    """
    Usage: python3 -m pup.http [METHOD] URL

    Where
        - METHOD is `get`. (DEFAULT: get.)
            - TODO: Support literally anything else.
        - URL is the URL to request.
    """

    if argv is None:
        argv = sys.argv

    args = argv[1:] # Drop the process name.

    if len(args) < 1 or len(args) > 2 or "-h" in args or "--help" in args:
        print(inspect.getdoc(_main), file=sys.stderr)
        return 1

    if len(args) == 2:
        method = args[0]
        url = args[1]
    else:
        method = "get"
        url = args[0]

    contents = fetch(method, url).read().decode()
    print(contents)

    return 0

if __name__ == "__main__":
    exit(_main())
